@@ -1,22 +1,25 @@
 """
 TkEasyGUI dialogs
 """
+import os
 import subprocess
 import tkinter.filedialog as filedialog
 import tkinter.messagebox as messagebox
 from datetime import datetime, timedelta
-from pprint import pprint
 from tkinter import colorchooser
-from typing import Any, Union
+from typing import Any, Callable, Iterable, Optional, Union
 
 from . import locale_easy as le
 from . import widgets as eg
-from .utils import (
-    FontType,
-    get_root_window,
-    is_mac,
-    is_win,
-)
+from .utils import ColorFormatType, FontType, get_root_window, is_mac, is_win
+
+YES = "Yes"
+NO = "No"
+OK = "OK"
+CANCEL = "Cancel"
+
+# save original print
+_print = print
 
 #------------------------------------------------------------------------------
 # Dialogs
@@ -29,7 +32,8 @@ def popup_buttons(
         buttons: list[str] = ["OK", "Cancel"], 
         auto_close_duration: int = -1, # auto close duration (msec)
         timeout_key: str="-TIMEOUT-", # timeout key
-        non_blocking: bool = False
+        non_blocking: bool = False,
+        default: str = '',
     ) -> str:
     """
     Popup window with user defined buttons. Return button's label.
@@ -45,13 +49,13 @@ def popup_buttons(
     """
     if title is None:
         title = le.get_text("Question")
-    result = buttons[-1] if len(buttons) > 0 else None
-    
+    result = buttons[-1] if len(buttons) > 0 else default
+
     # create window
-    win = eg.Window(title, layout=[
+    layout: eg.LayoutType = [
         [eg.Text(message)],
         [eg.Button(s, width=9) for s in buttons],
-    ], modal=True)
+    ]
     
     # event loop
     timer_id = eg.time_checker_start()
@@ -59,18 +63,15 @@ def popup_buttons(
     if non_blocking:
         # TODO: popup non blocking window
         pass
-    while win.is_alive():
-        event, _ = win.read(timeout=100, timeout_key=eg.WINDOW_TIMEOUT)
-        if event == eg.WINDOW_CLOSED:
-            result = eg.WINDOW_CLOSED
-        if event in buttons:
-            result = event
-            break
-        if event == eg.WINDOW_TIMEOUT:
-            if auto_close_duration > 0 and eg.time_checker_end(timer_id) > autoclose_sec:
-                result = timeout_key # timeout_key only use result
+    with eg.Window(title, layout=layout, modal=True) as win:
+        for event, _ in win.event_iter(timeout=100, timeout_key=eg.WINDOW_TIMEOUT):
+            if event in buttons:
+                result = event
                 break
-    win.close()
+            if event == eg.WINDOW_TIMEOUT:
+                if auto_close_duration > 0 and eg.time_checker_end(timer_id) > autoclose_sec:
+                    result = timeout_key # timeout_key only use result
+                    break
     return result
 
 def popup(message: str, title: str = "") -> str:
@@ -204,38 +205,54 @@ def popup_get_text(
         title: Union[str,None] = None,
         default: Union[str, None] = None,
         default_text: Union[str, None] = None, # same as default for compatibility
-        font: FontType=None) -> Union[str, None]:
+        font: Optional[FontType]=None) -> Union[str, None]:
     """Display a message in a popup window with a text entry. Return the text entered."""
     # return simpledialog.askstring(title, message, initialvalue=default)
     if default_text is not None:
         default = default_text
-    return popup_input(message, title, default, font=font)
+    default_str = ''
+    if default is not None:
+        default_str = default
+    result: Union[str, float, None] = popup_input(
+        message, title, default_str, font=font
+    )
+    if result is None:
+        return None
+    return str(result)
 
 def popup_input(
-        message: str,
-        title: Union[str,None] = None,
-        default: str = "",
-        ok_label: Union[str, None] = None,
-        cancel_label: Union[str, None] = None,
-        cancel_value: Any = None,
-        font: FontType=None) -> Union[str, None]:
+    message: str,
+    title: Optional[str] = None,
+    default: str = "",
+    ok_label: Optional[str] = None,
+    cancel_label: Optional[str] = None,
+    cancel_value: Any = None,
+    only_number: bool = False,
+    font: Optional[FontType] = None,
+) -> Union[str, float, None]:
     """Display a message in a popup window with a text entry. Return the text entered. if canceled, return cancel_value."""
     result = cancel_value
     if title is None:
-        title = le.get_text("Text input")
+        title = le.get_text("Text input") if only_number is False else le.get_text("Number input")
     if ok_label is None:
         ok_label = le.get_text("OK")
     if cancel_label is None:
         cancel_label = le.get_text("Cancel")
     win = eg.Window(title, layout=[
         [eg.Text(message)],
-        [eg.Input(default, key="-user-", width=40)],
+        [eg.Input(default, key="-user-", width=40, enable_events=True)],
         [eg.Button(ok_label, width=11), eg.Button(cancel_label, width=9)]
     ], modal=True, font=font)
     while True:
         event, values = win.read()
-        if event == ok_label:
+        if (event == ok_label) or ((event == "-user-") and (values["event_type"] == "return")):
             result = values["-user-"]
+            if only_number:
+                try:
+                    result = float(result)
+                except ValueError as _:
+                    popup(le.get_text("Please enter a number."))
+                    continue
             break
         if event in [cancel_label, eg.WINDOW_CLOSED]:
             # let result = cancel_value
@@ -266,46 +283,74 @@ def popup_info(message: str, title: Union[str,None]=None) -> None:
 def popup_get_file(
         message: str="",
         title: Union[str, None] = None,
-        initial_folder: str = "",
+        initial_folder: Union[str, None] = None,
         save_as: bool = False, # show `save as` dialog
         multiple_files: bool = False, # can select multiple files
         file_types: tuple[tuple[str, str]] = (("All Files", "*.*"),),
+        default_extension: Union[str, None] = None,
         no_window: Union[bool, None] = None, # for compatibility
         **kw) -> Union[str, tuple[str], None]:
     """Popup a file selection dialog. Return the file selected."""
     if title is None:
         title = message
+    if initial_folder is None:
+        initial_folder = os.getcwd()
+    file_type_list: Iterable[tuple[str, str | list[str] | tuple[str, ...]]] | None = []
+    if file_types is not None:
+        # check file types
+        new_types = []
+        for ft in file_types:
+            if len(ft) != 2:
+                ft = (ft, ft)
+            # fix file types for mac (#52)
+            # like ("Image Files", *.jpg;*.jpeg;*.jpe;*.heic")
+            if is_mac():
+                if ";" in ft[1]:
+                    desc = ft[0]
+                    exts = ft[1].split(";")
+                    for ext in exts:
+                        new_types.append((desc, ext))
+                    continue
+            new_types.append(ft)
+        file_type_list = tuple(new_types)
     if save_as:
         result = filedialog.asksaveasfilename(
             title=title,
             initialdir=initial_folder,
-            filetypes=file_types,
-            **kw)
+            filetypes=file_type_list,
+            defaultextension=default_extension,
+        )
+        if result and default_extension:
+            if result.endswith(default_extension) is False:
+                result += default_extension
     else:
         result = filedialog.askopenfilename(
-            title=title, 
+            title=title,
             initialdir=initial_folder,
-            filetypes=file_types,
-            multiple=multiple_files, # type: ignore
-            **kw)
+            filetypes=file_type_list,
+            multiple=multiple_files,  # type: ignore
+            **kw,
+        )
     return result
 
 def popup_get_folder(
             message: str = "",
             title: Union[str, None] = None,
-            default_path: str = "",
+            default_path: Union[str, None] = None,
             no_window: Union[bool, None] = None, # for compatibility
             **kw
             ) -> Union[str, None]:
     """Popup a folder selection dialog. Return the folder selected."""
     if title is None:
         title = message
+    if default_path is None:
+        default_path = os.getcwd()
     return filedialog.askdirectory(title=title, initialdir=default_path, **kw)
 
 def popup_memo(
         message: str,
         title: Union[str, None] = None,
-        size: tuple[int,int] = [60, 8],
+        size: tuple[int,int] = (60, 8),
         readonly: bool = False,
         ok_label: Union[str, None] = None,
         cancel_label: Union[str, None] = None,
@@ -316,15 +361,15 @@ def popup_memo(
     return popup_scrolled(message, title, size, readonly, ok_label, cancel_label, cancel_value, font)
 
 def popup_scrolled(
-            message: str,
-            title: tuple[str, None] = None,
-            size: tuple[int,int] = [40, 5],
-            readonly: bool = False,
-            ok_label: Union[str, None] = None,
-            cancel_label: Union[str, None] = None,
-            cancel_value: Union[str,None] = None,
-            font: Union[FontType, None] = None
-            ) -> Union[str, None]:
+    message: str,
+    title: Optional[str] = None,
+    size: tuple[int, int] = (40, 5),
+    readonly: bool = False,
+    ok_label: Optional[str] = None,
+    cancel_label: Optional[str] = None,
+    cancel_value: Optional[str] = None,
+    font: Optional[FontType] = None,
+) -> Union[str, None]:
     """
     Display a multiline message in a popup window. Return the text entered. if canceled, return cancel_value.
     #### Example:
@@ -339,12 +384,14 @@ def popup_scrolled(
         cancel_label = le.get_text("Cancel")
     if ok_label is None:
         ok_label = le.get_text("OK")
+    title_str: str = ''
     if title is None:
-        title = le.get_text("Information")
-    win = eg.Window(title, layout=[
+        title_str = le.get_text("Information")
+    layout: eg.LayoutType = [
         [eg.Multiline(message, key="-text-", size=size, readonly=readonly, font=font)],
-        [eg.Button(ok_label, width=9), eg.Button(cancel_label, width=5)]
-    ], modal=True)
+        [eg.Button(ok_label, width=9), eg.Button(cancel_label, width=5)],
+    ]
+    win: eg.Window = eg.Window(title=title_str, layout=layout, modal=True)
     result = None
     while win.is_alive():
         event, _ = win.read()
@@ -364,6 +411,7 @@ def popup_get_date(
         ok_label: Union[str, None] = None,
         cancel_label: Union[str, None] = None,
         date_format: Union[str, None] = None,
+        close_when_date_chosen: bool = False,
         sunday_first: bool = False, # Sunday is the first day of the week
         ) -> Union[datetime, None]:
     """Display a calendar in a popup window. Return the datetime entered or None."""
@@ -371,6 +419,7 @@ def popup_get_date(
         current_date = datetime.now()
     if date_format is None:
         date_format = le.get_text("__date_format__", "%Y-%m-%d")
+    date_format_str: str = date_format
     if title is None:
         title = le.get_text("Select date")
     # week names
@@ -383,22 +432,22 @@ def popup_get_date(
     head_size = [4, 1]
     # set
     result = None
-    layout = []
+    layout: eg.LayoutType = []
     # header
     if message != "":
         layout.append([eg.Text(message)])
         layout.append([eg.HSeparator()])
     # select month
-    month_button = [
+    month_button: list[eg.Element] = [
         eg.Frame(
             "",
             layout=[
                 [
-                    eg.Button("ᐊ", key="-prev2-", padx=0, size=[1, 1]),
-                    eg.Button("◀", key="-prev-", padx=0, size=[1, 1]),
-                    eg.Button(current_date.strftime(date_format), key="-ymd-", size=[10, 1]),
-                    eg.Button("▶", key="-next-", padx=0, size=[1, 1]),
-                    eg.Button("ᐅ", key="-next2-", padx=0, size=[1, 1]),
+                    eg.Button("ᐊ", key="-prev2-", padx=0, size=(1, 1)),
+                    eg.Button("◀", key="-prev-", padx=0, size=(1, 1)),
+                    eg.Button(current_date.strftime(date_format_str), key="-ymd-", size=(10, 1)),
+                    eg.Button("▶", key="-next-", padx=0, size=(1, 1)),
+                    eg.Button("ᐅ", key="-next2-", padx=0, size=(1, 1)),
                 ]
             ],
             expand_x=True,
@@ -408,7 +457,7 @@ def popup_get_date(
         )
     ]
     layout.append(month_button)
-    week_labels = [
+    week_labels: list[eg.Element] = [
         eg.Text(
             name,
             size=head_size,
@@ -438,12 +487,12 @@ def popup_get_date(
                 top_date = first_day - timedelta(days=week_i)
         return top_date
     # create calendar days buttons
-    cols = []
+    cols: list[eg.Element] = []
     cur = get_top_date(current_date)
     for i in range(42):
         cols.append(
             eg.Text(
-                cur.day,
+                f"{cur.day}",
                 size=days_size,
                 key=f"-b{i}-",
                 text_color=week_colors[i % 7],
@@ -466,7 +515,19 @@ def popup_get_date(
         ok_label = le.get_text("OK")
     if cancel_label is None:
         cancel_label = le.get_text("Cancel")
-    layout.append([eg.Button(ok_label, size=(8, 1)), eg.Button(cancel_label, size=(5, 1))])
+    panel = eg.Frame(
+        "",
+        layout=[
+            [
+                eg.Button(ok_label, size=(10, 1)),
+                eg.Button(cancel_label, size=(8, 1)),
+            ]
+        ],
+        text_align="center",
+        expand_x=True,
+        relief="flat",
+    )
+    layout.append([panel])
     # update label
     def update_date(top: datetime, current_date: datetime):
         cur = datetime(year=top.year, month=top.month, day=top.day)
@@ -482,13 +543,14 @@ def popup_get_date(
                 if i % 7 == 6:
                     fg = "black"
             # update
-            btn.set_text(cur.day)
+            btn.set_text(str(cur.day))
             btn.set_disabled(current_date.month != cur.month)
             btn.update(text_color=fg, background_color=bg)
-            btn.metadata["date"] = datetime.fromtimestamp(cur.timestamp())
+            if btn.metadata is not None:
+                btn.metadata["date"] = datetime.fromtimestamp(cur.timestamp())
             # next day
             cur = cur + timedelta(days=1)
-        window["-ymd-"].update(current_date.strftime(date_format))
+        window["-ymd-"].update(current_date.strftime(date_format_str))
     # update result
     def update_result(current_date):
         top = get_top_date(datetime(year=current_date.year, month=current_date.month, day=1))
@@ -530,17 +592,150 @@ def popup_get_date(
                 update_date(get_top_date(current_date), current_date)
             elif event.startswith("-b"):
                 elm:eg.Text = window[event]
-                if elm is None:
+                if elm is None or elm.metadata is None:
                     continue
                 sel_date: datetime = elm.metadata["date"]
                 if sel_date.month == current_date.month:
                     current_date = sel_date
                     update_result(current_date)
+                    if close_when_date_chosen:
+                        result = current_date
+                        break
                 else:
                     current_date = sel_date
                     update_date(get_top_date(current_date), current_date)
     return result
 
+def popup_get_form(
+    form_items: list[Union[str, tuple[str, Any], tuple[str, Any, str]]], # list of form items(label[,selection or default][,type])
+    title: str = "Form" # window title
+) -> Union[dict[str, Any], None]:
+    """
+        Displays a form that allows multiple items to be entered.
+        By specifying the labels and input types for each item, the form is automatically generated and displayed in a dialog.
+        When the user enters the items and clicks [OK], it returns `{label: value}`. If the user clicks [Cancel], it returns `None`.
+
+        The `form_items` argument can specify input fields simply as `[label1, label2, label3, ...]`.
+        Alternatively, you can specify labels, types, and default values as `[(label1, default1, type1), (label2, default2, type2), ...]`.
+        The following values can be specified for the type: `text`, `number`, `password`, `combo`, `list`, `date`, `file`, `files`, `folder`, `color`.
+
+        @see [tests/popup_get_form_test.py](/tests/popup_get_form_test.py)
+
+        #### Simple Example:
+        ```py
+        import TkEasyGUI as eg
+        form = eg.popup_get_form(["Name", "Hobbies"])
+        if form:
+            name = form["Name"]
+            hobbies = form["Hobbies"]
+            eg.print(f"name={name}, hobbies={hobbies}")
+        ```
+
+        #### Example:
+        ```py
+        import TkEasyGUI as eg
+        result = eg.popup_get_form([
+            # [label, default_or_selections, type]
+            ["Name", "Suzu", "text"],
+            ["Age", 20, "number"],
+            ["Hobbies", ["Reading", "Writing"], "list"],
+            ["OS", ["Windows", "macOS", "Ubuntu"], "combo"],
+            ["Password", "", "password"],
+            ["Date", "", "date"],
+            ["File", "", "file"],
+            ["Folder", "", "folder"],
+            ["Theme Color", "", "color"],
+        ])
+        if result:
+            print(result)
+        ```
+    """
+    # make form layout
+    item_labels = []
+    item_converters: list[Union[Callable, None]] = []
+    layout: eg.LayoutType = []
+    for i, it in enumerate(form_items):
+        it_key = f"-formitem{i}"
+        if isinstance(it, tuple) or isinstance(it, list):
+            label = it[0]
+            default_value = it[1] if len(it) >= 2 else ""
+            itype = it[2] if len(it) >= 3 else "text"
+            if isinstance(default_value, tuple) or isinstance(default_value, list):
+                itype = "list"
+        else:
+            label = it
+            default_value = ""
+            itype = "text"
+        if itype == "":
+            itype = "text"
+        # for making result
+        item_labels.append(label)
+        item_converters.append(None)
+        # make line
+        line: list[eg.Element] = [eg.Text(label, key=f"{it_key}-label", size=(15, 1), text_align="right")]
+        # check type
+        itype = itype.lower()
+        if itype == "text" or itype == "number" or itype == "password":
+            line.append(eg.Input(
+                default_value, key=it_key, size=(20, 1),
+                password_char="*" if itype == "password" else None))
+            if itype == "number":
+                item_converters[i] = float
+        elif itype == "combo":
+            sels = default_value
+            val = sels[0] if len(sels) > 0 else ""
+            line.append(eg.Combo(sels, default_value=val, key=it_key, size=(19, 1)))
+        elif itype == "list":
+            sels = default_value
+            val = sels[0] if len(sels) > 0 else ""
+            line.append(eg.Input(val, key=it_key, size=(15, 1)))
+            line.append(eg.ListBrowse(sels))
+        elif itype == "date":
+            line.append(eg.Input(default_value, key=it_key, size=(15, 1)))
+            line.append(eg.CalendarBrowse())
+        elif itype == "file":
+            line.append(eg.Input(default_value, key=it_key, size=(15, 1)))
+            line.append(eg.FileBrowse())
+        elif itype == "files":
+            line.append(eg.Input(default_value, key=it_key, size=(15, 1)))
+            line.append(eg.FileBrowse(multiple_files=True))
+        elif itype == "folder":
+            line.append(eg.Input(default_value, key=it_key, size=(15, 1)))
+            line.append(eg.FolderBrowse())
+        elif itype == "color":
+            line.append(eg.Input(default_value, key=it_key, size=(15, 1)))
+            line.append(eg.ColorBrowse())
+        # append line
+        layout.append(line)
+    layout.append([eg.HSeparator()])
+    layout.append([eg.Button("OK", width=9), eg.Button("Cancel", width=5)])
+    with eg.Window(title, layout=layout) as win:
+        result: Union[dict[str, Any], None] = None
+        for event, values in win.event_iter():
+            if event == "Cancel":
+                result = None
+                break
+            if event == "OK":
+                result = {}
+                for i, label in enumerate(item_labels):
+                    it_key = f"-formitem{i}"
+                    val = values[it_key] if it_key in values else ""
+                    fn = item_converters[i]
+                    try:
+                        if fn is not None:
+                            val = fn(val)
+                    except ValueError as _:
+                        popup(le.get_text("Please enter a number."))
+                        win.focus_element(it_key)
+                        result = None
+                        break
+                    result[label] = val
+                # form error?
+                if result is None:
+                    continue
+                break
+    return result
+
 #------------------------------------------------------------------------------
 # for notify
 #------------------------------------------------------------------------------
@@ -584,43 +779,64 @@ $AppId = '{{1AC14E77-02E7-4E5D-B744-2EB1AE5198B7}}\WindowsPowerShell\v1.0\powers
 # TkEasyGUI original dialogs
 #------------------------------------------------------------------------------
 
-def popup_color(title: str="", default_color: Union[str, None]=None) -> (Union[str, None]):
-    """Popup a color selection dialog. Return the color selected."""
+def popup_color(
+    title: str = "",
+    default_color: Union[str, None] = None,
+    format: ColorFormatType = "html",
+) -> Union[str, tuple[int,int,int], None]:
+    """
+    Popup a color selection dialog. Return the color selected.
+    format: "html", "rgb", "tuple"
+    """
     col = colorchooser.askcolor(title=title, color=default_color)
     if col[1] is None:
         return default_color
-    return f"{col[1]}".upper()
+    html = col[1].upper()
+    if format == "html":
+        return html
+    elif format == "rgb":
+        return html[1:]
+    elif format == "tuple":
+        return col[0]
+    return html
 
 def popup_listbox(
-        items: list[str], # list of items
-        message: str = "",
-        title: str = "",
-        size: tuple[int,int] = (20, 7),
-        font: Union[FontType, None] = None,
-        multiple:bool = False # multiple selection
-        ) -> Union[str, None]:
+    values: list[str],  # list of items
+    message: str = "",
+    title: str = "",
+    size: tuple[int, int] = (20, 7),
+    font: Union[FontType, None] = None,
+    default_value: Union[str, None] = None,  # default value
+    multiple: bool = False,  # multiple selection
+) -> Union[str, None]:
     """Display Listbox in a popup window"""
-    select_mode = eg.LISTBOX_SELECT_MODE_BROWSE if multiple is False else eg.LISTBOX_SELECT_MODE_MULTIPLE
-    win = eg.Window(title, layout=[
-        [eg.Text(message)],
-        [eg.Listbox(values=items, key="-list-", size=size, font=font, select_mode=select_mode)],
-        [eg.Button("OK", width=9), eg.Button("Cancel", width=5)]
-    ], modal=True)
-    result = None
-    while win.is_alive():
-        event, _ = win.read()
-        if event == "Cancel":
-            result = None
-            break
-        if event == "OK":
-            selected = win["-list-"].get()
-            if multiple:
-                result = selected
-            else:
-                if len(selected) == 1:
-                    result = selected[0]
-            break
-    win.close()
+    select_mode: eg.ListboxSelectMode = eg.LISTBOX_SELECT_MODE_BROWSE if multiple is False else eg.LISTBOX_SELECT_MODE_MULTIPLE
+    # create window
+    layout: list[list[eg.Element]] = []
+    if message != "":
+        layout.append([eg.Text(message)])
+    layout.append([
+        eg.Listbox(
+            values=values, key="-list-",
+            default_value=default_value, size=size,
+            font=font, select_mode=select_mode
+        )])
+    layout.append([eg.Button("OK", width=9), eg.Button("Cancel", width=5)])
+    with eg.Window(title, layout=layout, modal=True) as win:
+        # event loop
+        result = None
+        for event, values in win.event_iter():
+            if event == "Cancel":
+                result = None
+                break
+            if event == "OK":
+                selected = win["-list-"].get()
+                if multiple:
+                    result = selected
+                else:
+                    if len(selected) == 1:
+                        result = selected[0]
+                break
     return result
 
 def popup_image(
@@ -632,21 +848,22 @@ def popup_image(
         ok_label: Union[str, None] = None,
         ok_value: str = "OK",
         cancel_label: Union[str, None] = None,
-        cancel_value: str = "Cancel",
+        cancel_value: Union[str, None] = None,
         font: Union[FontType, None] = None,
-        ) -> str:
-    """Display an image in a popup window. Return the text entered."""
+        ) -> Union[str, None]:
+    """Display an image in a popup window. Return the pushed Button("OK" or None)."""
     if title is None:
         title = message
     if ok_label is None:
         ok_label = le.get_text("OK")
     if cancel_label is None:
         cancel_label = le.get_text("Cancel")
-    win = eg.Window(title, layout=[
+    layout: list[list[eg.Element]] = [
         [eg.Text(message)],
         [eg.Image(image_path, image_data, size=size)],
-        [eg.Button(ok_label, width=9), eg.Button(cancel_label, width=5)]
-    ], modal=True, font=font)
+        [eg.Button(ok_label, width=9), eg.Button(cancel_label, width=5)],
+    ]
+    win = eg.Window(title, layout=layout, modal=True, font=font)
     result = cancel_value
     while win.is_alive():
         event, _ = win.read()
@@ -704,16 +921,31 @@ def msgbox(
 def input(
         message: str,
         title: Union[str,None] = None,
-        default: str = ""
-    ) -> str:
+        default: str = "",
+        only_number: bool = False
+    ) -> Union[str, float, None]:
     """Display a message in a popup window with a text entry. Return the text entered."""
-    return popup_input(message, title, default)
+    return popup_input(message, title, default, only_number=only_number)
 
 def print(*args, **kw) -> None:
-    """Print message to popup window."""
+    """Print message to popup window.(call default print function if no_window is True)"""
+    if "no_window" in kw and kw["no_window"]:
+        _print(*args)
+        return
     lines = " ".join([str(a) for a in args])
     popup(lines)
 
+def input_number(
+        message: str,
+        title: Union[str,None] = None,
+        default: str = ""
+    ) -> Union[float, None]:
+    """Display a message in a popup window with a number entry. Return the text entered."""
+    result = popup_input(message, title, default, only_number=True)
+    if isinstance(result, float):
+        return result
+    return None
+
 def confirm(
         question: str,
         title: Union[str,None] = None
